"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _hoistNonReactStatics = _interopRequireDefault(require("hoist-non-react-statics"));

var _lodash = _interopRequireDefault(require("lodash.difference"));

var _lodash2 = _interopRequireDefault(require("lodash.intersection"));

var _react = _interopRequireDefault(require("react"));

var _reactRedux = require("react-redux");

var _actions = require("redux-query/src/actions");

var _queryKey = require("redux-query/src/lib/query-key");

var _useConstCallback = _interopRequireDefault(require("../hooks/use-const-callback"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; var ownKeys = Object.keys(source); if (typeof Object.getOwnPropertySymbols === 'function') { ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) { return Object.getOwnPropertyDescriptor(source, sym).enumerable; })); } ownKeys.forEach(function (key) { _defineProperty(target, key, source[key]); }); } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

const normalizeToArray = maybe => {
  return (Array.isArray(maybe) ? maybe : [maybe]).filter(Boolean);
};

const getDiff = (prevActions, actions) => {
  const prevQueryKeys = prevActions.map(_queryKey.getQueryKey);
  const queryKeys = actions.map(_queryKey.getQueryKey);
  const actionByQueryKey = queryKeys.reduce((accum, queryKey, i) => {
    accum.set(queryKey, actions[i]);
    return accum;
  }, new Map());
  const intersect = (0, _lodash2.default)(prevQueryKeys, queryKeys);
  const cancelKeys = (0, _lodash.default)(prevQueryKeys, intersect);
  const requestKeys = (0, _lodash.default)(queryKeys, intersect);
  const requestActions = requestKeys.map(queryKey => actionByQueryKey.get(queryKey));
  return {
    cancelKeys,
    requestActions
  };
};

const useMemoizedActions = (mapPropsToConfigs, props, callback) => {
  const actions = normalizeToArray(mapPropsToConfigs(props)).map(queryConfig => _objectSpread({}, queryConfig, {
    unstable_preDispatchCallback: () => callback((0, _queryKey.getQueryKey)(queryConfig))
  }));

  const [memoizedActions, setMemoizedActions] = _react.default.useState(actions);

  const previousQueryKeys = _react.default.useRef(actions.map(_queryKey.getQueryKey));

  _react.default.useEffect(() => {
    const queryKeys = actions.map(_queryKey.getQueryKey);

    if (queryKeys.length !== previousQueryKeys.current.length || queryKeys.some((queryKey, i) => previousQueryKeys.current[i] !== queryKey)) {
      previousQueryKeys.current = queryKeys;
      setMemoizedActions(actions);
    }
  }, [actions]);

  return memoizedActions;
};

const useMultiRequest = (mapPropsToConfigs, props) => {
  const reduxDispatch = (0, _reactRedux.useDispatch)();

  const previousActions = _react.default.useRef([]);

  const pendingRequests = _react.default.useRef(new Set());

  const dispatchRequestToRedux = (0, _useConstCallback.default)(action => {
    const promise = reduxDispatch((0, _actions.requestAsync)(action));

    if (promise) {
      pendingRequests.current.add((0, _queryKey.getQueryKey)(action));
    }
  });
  const dispatchCancelToRedux = (0, _useConstCallback.default)(action => {
    reduxDispatch((0, _actions.cancelQuery)(action));
    pendingRequests.current.delete((0, _queryKey.getQueryKey)(action));
  });
  const requestReduxActions = useMemoizedActions(mapPropsToConfigs, props, queryKey => {
    pendingRequests.current.delete(queryKey);
  });

  _react.default.useEffect(() => {
    const {
      cancelKeys,
      requestActions
    } = getDiff(previousActions.current, requestReduxActions);
    requestActions.forEach(dispatchRequestToRedux);
    cancelKeys.forEach(dispatchCancelToRedux);
    previousActions.current = requestReduxActions;
    return () => {
      [...pendingRequests.current].forEach(dispatchCancelToRedux);
    };
  }, [dispatchCancelToRedux, dispatchRequestToRedux, requestReduxActions]);

  const forceRequest = _react.default.useCallback(() => {
    requestReduxActions.forEach(requestReduxAction => {
      dispatchRequestToRedux(_objectSpread({}, requestReduxAction, {
        force: true
      }));
    });
  }, [dispatchRequestToRedux, requestReduxActions]);

  return forceRequest;
};

const connectRequest = (mapPropsToConfigs, options = {}) => WrappedComponent => {
  const {
    pure = true,
    forwardRef = false
  } = options;

  const ConnectRequestFunction = props => {
    const forceRequest = useMultiRequest(mapPropsToConfigs, props);
    return _react.default.createElement(WrappedComponent, _extends({}, props, {
      forceRequest: forceRequest
    }));
  };

  const ConnectRequest = pure ? _react.default.memo(ConnectRequestFunction) : ConnectRequestFunction;
  const wrappedComponentName = WrappedComponent.displayName || WrappedComponent.name || 'Component';
  const displayName = `ConnectRequest(${wrappedComponentName})`;
  ConnectRequest.WrappedComponent = WrappedComponent;
  ConnectRequest.displayName = displayName;

  if (forwardRef) {
    const forwarded = _react.default.forwardRef((props, ref) => _react.default.createElement(ConnectRequest, _extends({}, props, {
      forwardedRef: ref
    })));

    forwarded.displayName = displayName;
    forwarded.WrappedComponent = WrappedComponent;
    return (0, _hoistNonReactStatics.default)(forwarded, WrappedComponent);
  }

  return (0, _hoistNonReactStatics.default)(ConnectRequest, WrappedComponent);
};

var _default = connectRequest;
exports.default = _default;